# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome._utils import parse_datetime
from metronome.pagination import SyncCursorPage, AsyncCursorPage
from metronome.types.v1.customers import (
    PlanAddResponse,
    PlanEndResponse,
    PlanListResponse,
    PlanListPriceAdjustmentsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlans:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.customers.plans.with_raw_response.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.customers.plans.with_streaming_response.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.plans.with_raw_response.list(
                customer_id="",
            )

    @parametrize
    def test_method_add(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        )
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    def test_method_add_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
            ending_before=parse_datetime("2022-02-01T00:00:00Z"),
            net_payment_terms_days=0,
            overage_rate_adjustments=[
                {
                    "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "fiat_currency_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "to_fiat_conversion_factor": 0,
                }
            ],
            price_adjustments=[
                {
                    "adjustment_type": "percentage",
                    "charge_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "start_period": 0,
                    "quantity": 0,
                    "tier": 0,
                    "value": 0,
                }
            ],
            trial_spec={
                "length_in_days": 0,
                "spending_cap": {
                    "amount": 0,
                    "credit_type_id": "credit_type_id",
                },
            },
        )
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    def test_raw_response_add(self, client: Metronome) -> None:
        response = client.v1.customers.plans.with_raw_response.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    def test_streaming_response_add(self, client: Metronome) -> None:
        with client.v1.customers.plans.with_streaming_response.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanAddResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_add(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.plans.with_raw_response.add(
                customer_id="",
                plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
                starting_on=parse_datetime("2021-02-01T00:00:00Z"),
            )

    @parametrize
    def test_method_end(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    def test_method_end_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            ending_before=parse_datetime("2021-02-01T00:00:00Z"),
            void_invoices=True,
            void_stripe_invoices=True,
        )
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    def test_raw_response_end(self, client: Metronome) -> None:
        response = client.v1.customers.plans.with_raw_response.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    def test_streaming_response_end(self, client: Metronome) -> None:
        with client.v1.customers.plans.with_streaming_response.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanEndResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_end(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.plans.with_raw_response.end(
                customer_id="",
                customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_plan_id` but received ''"):
            client.v1.customers.plans.with_raw_response.end(
                customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
                customer_plan_id="",
            )

    @parametrize
    def test_method_list_price_adjustments(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )
        assert_matches_type(SyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    def test_method_list_price_adjustments_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.customers.plans.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(SyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    def test_raw_response_list_price_adjustments(self, client: Metronome) -> None:
        response = client.v1.customers.plans.with_raw_response.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    def test_streaming_response_list_price_adjustments(self, client: Metronome) -> None:
        with client.v1.customers.plans.with_streaming_response.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_price_adjustments(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.plans.with_raw_response.list_price_adjustments(
                customer_id="",
                customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_plan_id` but received ''"):
            client.v1.customers.plans.with_raw_response.list_price_adjustments(
                customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
                customer_plan_id="",
            )


class TestAsyncPlans:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.plans.with_raw_response.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.plans.with_streaming_response.list(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.list(
                customer_id="",
            )

    @parametrize
    async def test_method_add(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        )
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    async def test_method_add_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
            ending_before=parse_datetime("2022-02-01T00:00:00Z"),
            net_payment_terms_days=0,
            overage_rate_adjustments=[
                {
                    "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "fiat_currency_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "to_fiat_conversion_factor": 0,
                }
            ],
            price_adjustments=[
                {
                    "adjustment_type": "percentage",
                    "charge_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "start_period": 0,
                    "quantity": 0,
                    "tier": 0,
                    "value": 0,
                }
            ],
            trial_spec={
                "length_in_days": 0,
                "spending_cap": {
                    "amount": 0,
                    "credit_type_id": "credit_type_id",
                },
            },
        )
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    async def test_raw_response_add(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.plans.with_raw_response.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanAddResponse, plan, path=["response"])

    @parametrize
    async def test_streaming_response_add(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.plans.with_streaming_response.add(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
            starting_on=parse_datetime("2021-02-01T00:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanAddResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_add(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.add(
                customer_id="",
                plan_id="d2c06dae-9549-4d7d-bc04-b78dd3d241b8",
                starting_on=parse_datetime("2021-02-01T00:00:00Z"),
            )

    @parametrize
    async def test_method_end(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    async def test_method_end_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            ending_before=parse_datetime("2021-02-01T00:00:00Z"),
            void_invoices=True,
            void_stripe_invoices=True,
        )
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    async def test_raw_response_end(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.plans.with_raw_response.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanEndResponse, plan, path=["response"])

    @parametrize
    async def test_streaming_response_end(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.plans.with_streaming_response.end(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanEndResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_end(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.end(
                customer_id="",
                customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_plan_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.end(
                customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
                customer_plan_id="",
            )

    @parametrize
    async def test_method_list_price_adjustments(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )
        assert_matches_type(AsyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    async def test_method_list_price_adjustments_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.customers.plans.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(AsyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    async def test_raw_response_list_price_adjustments(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.plans.with_raw_response.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

    @parametrize
    async def test_streaming_response_list_price_adjustments(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.plans.with_streaming_response.list_price_adjustments(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListPriceAdjustmentsResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_price_adjustments(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.list_price_adjustments(
                customer_id="",
                customer_plan_id="7aa11640-0703-4600-8eb9-293f535a6b74",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_plan_id` but received ''"):
            await async_client.v1.customers.plans.with_raw_response.list_price_adjustments(
                customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
                customer_plan_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.service_list_response import ServiceListResponse

__all__ = ["ServicesResource", "AsyncServicesResource"]


class ServicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return ServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return ServicesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ServiceListResponse:
        """Fetches a list of services used by Metronome and the associated IP addresses.

        IP
        addresses are not necessarily unique between services. In most cases, IP
        addresses will appear in the list at least 30 days before they are used for the
        first time.
        """
        return self._get(
            "/v1/services",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceListResponse,
        )


class AsyncServicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncServicesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ServiceListResponse:
        """Fetches a list of services used by Metronome and the associated IP addresses.

        IP
        addresses are not necessarily unique between services. In most cases, IP
        addresses will appear in the list at least 30 days before they are used for the
        first time.
        """
        return await self._get(
            "/v1/services",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceListResponse,
        )


class ServicesResourceWithRawResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.list = to_raw_response_wrapper(
            services.list,
        )


class AsyncServicesResourceWithRawResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.list = async_to_raw_response_wrapper(
            services.list,
        )


class ServicesResourceWithStreamingResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.list = to_streamed_response_wrapper(
            services.list,
        )


class AsyncServicesResourceWithStreamingResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.list = async_to_streamed_response_wrapper(
            services.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1.contracts.rate_cards import (
    ProductOrderSetResponse,
    ProductOrderUpdateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProductOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: Metronome) -> None:
        product_order = client.v1.contracts.rate_cards.product_orders.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Metronome) -> None:
        response = client.v1.contracts.rate_cards.product_orders.with_raw_response.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product_order = response.parse()
        assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Metronome) -> None:
        with client.v1.contracts.rate_cards.product_orders.with_streaming_response.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product_order = response.parse()
            assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_set(self, client: Metronome) -> None:
        product_order = client.v1.contracts.rate_cards.product_orders.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

    @parametrize
    def test_raw_response_set(self, client: Metronome) -> None:
        response = client.v1.contracts.rate_cards.product_orders.with_raw_response.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product_order = response.parse()
        assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

    @parametrize
    def test_streaming_response_set(self, client: Metronome) -> None:
        with client.v1.contracts.rate_cards.product_orders.with_streaming_response.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product_order = response.parse()
            assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncProductOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_update(self, async_client: AsyncMetronome) -> None:
        product_order = await async_client.v1.contracts.rate_cards.product_orders.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.rate_cards.product_orders.with_raw_response.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product_order = await response.parse()
        assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.rate_cards.product_orders.with_streaming_response.update(
            product_moves=[
                {
                    "position": 0,
                    "product_id": "13117714-3f05-48e5-a6e9-a66093f13b4d",
                },
                {
                    "position": 1,
                    "product_id": "b086f2f4-9851-4466-9ca0-30d53e6a42ac",
                },
            ],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product_order = await response.parse()
            assert_matches_type(ProductOrderUpdateResponse, product_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_set(self, async_client: AsyncMetronome) -> None:
        product_order = await async_client.v1.contracts.rate_cards.product_orders.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

    @parametrize
    async def test_raw_response_set(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.rate_cards.product_orders.with_raw_response.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product_order = await response.parse()
        assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

    @parametrize
    async def test_streaming_response_set(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.rate_cards.product_orders.with_streaming_response.set(
            product_order=["13117714-3f05-48e5-a6e9-a66093f13b4d", "b086f2f4-9851-4466-9ca0-30d53e6a42ac"],
            rate_card_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product_order = await response.parse()
            assert_matches_type(ProductOrderSetResponse, product_order, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ...types.v1 import audit_log_list_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.audit_log_list_response import AuditLogListResponse

__all__ = ["AuditLogsResource", "AsyncAuditLogsResource"]


class AuditLogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        resource_id: str | NotGiven = NOT_GIVEN,
        resource_type: str | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[AuditLogListResponse]:
        """Retrieves a range of audit logs.

        If no further audit logs are currently
        available, the data array will be empty. As new audit logs are created,
        subsequent requests using the same next_page value will be in the returned data
        array, ensuring a continuous and uninterrupted reading of audit logs.

        Args:
          ending_before: RFC 3339 timestamp (exclusive). Cannot be used with 'next_page'.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          resource_id: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_id, you must also specify resource_type.

          resource_type: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_type, you must also specify resource_id.

          sort: Sort order by timestamp, e.g. date_asc or date_desc. Defaults to date_asc.

          starting_on: RFC 3339 timestamp of the earliest audit log to return. Cannot be used with
              'next_page'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/auditLogs",
            page=SyncCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "sort": sort,
                        "starting_on": starting_on,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )


class AsyncAuditLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncAuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        resource_id: str | NotGiven = NOT_GIVEN,
        resource_type: str | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[AuditLogListResponse, AsyncCursorPage[AuditLogListResponse]]:
        """Retrieves a range of audit logs.

        If no further audit logs are currently
        available, the data array will be empty. As new audit logs are created,
        subsequent requests using the same next_page value will be in the returned data
        array, ensuring a continuous and uninterrupted reading of audit logs.

        Args:
          ending_before: RFC 3339 timestamp (exclusive). Cannot be used with 'next_page'.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          resource_id: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_id, you must also specify resource_type.

          resource_type: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_type, you must also specify resource_id.

          sort: Sort order by timestamp, e.g. date_asc or date_desc. Defaults to date_asc.

          starting_on: RFC 3339 timestamp of the earliest audit log to return. Cannot be used with
              'next_page'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/auditLogs",
            page=AsyncCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "sort": sort,
                        "starting_on": starting_on,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )


class AuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_raw_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_raw_response_wrapper(
            audit_logs.list,
        )


class AuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_streamed_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_streamed_response_wrapper(
            audit_logs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .id import ID as ID
from .rate import Rate as Rate
from .tier import Tier as Tier
from .commit import Commit as Commit
from .credit import Credit as Credit
from .discount import Discount as Discount
from .override import Override as Override
from .pro_service import ProService as ProService
from .property_filter import PropertyFilter as PropertyFilter
from .credit_type_data import CreditTypeData as CreditTypeData
from .scheduled_charge import ScheduledCharge as ScheduledCharge
from .base_usage_filter import BaseUsageFilter as BaseUsageFilter
from .event_type_filter import EventTypeFilter as EventTypeFilter
from .schedule_duration import ScheduleDuration as ScheduleDuration
from .schedule_point_in_time import SchedulePointInTime as SchedulePointInTime
from .contract_without_amendments import ContractWithoutAmendments as ContractWithoutAmendments



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from .schedule_duration import ScheduleDuration
from .schedule_point_in_time import SchedulePointInTime

__all__ = [
    "Commit",
    "Product",
    "Contract",
    "HierarchyConfiguration",
    "HierarchyConfigurationChildAccess",
    "HierarchyConfigurationChildAccessCommitHierarchyChildAccessAll",
    "HierarchyConfigurationChildAccessCommitHierarchyChildAccessNone",
    "HierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs",
    "InvoiceContract",
    "Ledger",
    "LedgerPrepaidCommitSegmentStartLedgerEntry",
    "LedgerPrepaidCommitAutomatedInvoiceDeductionLedgerEntry",
    "LedgerPrepaidCommitRolloverLedgerEntry",
    "LedgerPrepaidCommitExpirationLedgerEntry",
    "LedgerPrepaidCommitCanceledLedgerEntry",
    "LedgerPrepaidCommitCreditedLedgerEntry",
    "LedgerPrepaidCommitSeatBasedAdjustmentLedgerEntry",
    "LedgerPostpaidCommitInitialBalanceLedgerEntry",
    "LedgerPostpaidCommitAutomatedInvoiceDeductionLedgerEntry",
    "LedgerPostpaidCommitRolloverLedgerEntry",
    "LedgerPostpaidCommitTrueupLedgerEntry",
    "LedgerPrepaidCommitManualLedgerEntry",
    "LedgerPostpaidCommitManualLedgerEntry",
    "LedgerPostpaidCommitExpirationLedgerEntry",
    "RolledOverFrom",
    "Specifier",
]


class Product(BaseModel):
    id: str

    name: str


class Contract(BaseModel):
    id: str


class HierarchyConfigurationChildAccessCommitHierarchyChildAccessAll(BaseModel):
    type: Literal["ALL"]


class HierarchyConfigurationChildAccessCommitHierarchyChildAccessNone(BaseModel):
    type: Literal["NONE"]


class HierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs(BaseModel):
    contract_ids: List[str]

    type: Literal["CONTRACT_IDS"]


HierarchyConfigurationChildAccess: TypeAlias = Union[
    HierarchyConfigurationChildAccessCommitHierarchyChildAccessAll,
    HierarchyConfigurationChildAccessCommitHierarchyChildAccessNone,
    HierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs,
]


class HierarchyConfiguration(BaseModel):
    child_access: HierarchyConfigurationChildAccess


class InvoiceContract(BaseModel):
    id: str


class LedgerPrepaidCommitSegmentStartLedgerEntry(BaseModel):
    amount: float

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_SEGMENT_START"]


class LedgerPrepaidCommitAutomatedInvoiceDeductionLedgerEntry(BaseModel):
    amount: float

    invoice_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_AUTOMATED_INVOICE_DEDUCTION"]

    contract_id: Optional[str] = None


class LedgerPrepaidCommitRolloverLedgerEntry(BaseModel):
    amount: float

    new_contract_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_ROLLOVER"]


class LedgerPrepaidCommitExpirationLedgerEntry(BaseModel):
    amount: float

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_EXPIRATION"]


class LedgerPrepaidCommitCanceledLedgerEntry(BaseModel):
    amount: float

    invoice_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_CANCELED"]

    contract_id: Optional[str] = None


class LedgerPrepaidCommitCreditedLedgerEntry(BaseModel):
    amount: float

    invoice_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_CREDITED"]

    contract_id: Optional[str] = None


class LedgerPrepaidCommitSeatBasedAdjustmentLedgerEntry(BaseModel):
    amount: float

    segment_id: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_SEAT_BASED_ADJUSTMENT"]


class LedgerPostpaidCommitInitialBalanceLedgerEntry(BaseModel):
    amount: float

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_INITIAL_BALANCE"]


class LedgerPostpaidCommitAutomatedInvoiceDeductionLedgerEntry(BaseModel):
    amount: float

    invoice_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_AUTOMATED_INVOICE_DEDUCTION"]

    contract_id: Optional[str] = None


class LedgerPostpaidCommitRolloverLedgerEntry(BaseModel):
    amount: float

    new_contract_id: str

    segment_id: str

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_ROLLOVER"]


class LedgerPostpaidCommitTrueupLedgerEntry(BaseModel):
    amount: float

    invoice_id: str

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_TRUEUP"]

    contract_id: Optional[str] = None


class LedgerPrepaidCommitManualLedgerEntry(BaseModel):
    amount: float

    reason: str

    timestamp: datetime

    type: Literal["PREPAID_COMMIT_MANUAL"]


class LedgerPostpaidCommitManualLedgerEntry(BaseModel):
    amount: float

    reason: str

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_MANUAL"]


class LedgerPostpaidCommitExpirationLedgerEntry(BaseModel):
    amount: float

    timestamp: datetime

    type: Literal["POSTPAID_COMMIT_EXPIRATION"]


Ledger: TypeAlias = Union[
    LedgerPrepaidCommitSegmentStartLedgerEntry,
    LedgerPrepaidCommitAutomatedInvoiceDeductionLedgerEntry,
    LedgerPrepaidCommitRolloverLedgerEntry,
    LedgerPrepaidCommitExpirationLedgerEntry,
    LedgerPrepaidCommitCanceledLedgerEntry,
    LedgerPrepaidCommitCreditedLedgerEntry,
    LedgerPrepaidCommitSeatBasedAdjustmentLedgerEntry,
    LedgerPostpaidCommitInitialBalanceLedgerEntry,
    LedgerPostpaidCommitAutomatedInvoiceDeductionLedgerEntry,
    LedgerPostpaidCommitRolloverLedgerEntry,
    LedgerPostpaidCommitTrueupLedgerEntry,
    LedgerPrepaidCommitManualLedgerEntry,
    LedgerPostpaidCommitManualLedgerEntry,
    LedgerPostpaidCommitExpirationLedgerEntry,
]


class RolledOverFrom(BaseModel):
    commit_id: str

    contract_id: str


class Specifier(BaseModel):
    presentation_group_values: Optional[Dict[str, str]] = None

    pricing_group_values: Optional[Dict[str, str]] = None

    product_id: Optional[str] = None
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: Optional[List[str]] = None
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """


class Commit(BaseModel):
    id: str

    product: Product

    type: Literal["PREPAID", "POSTPAID"]

    access_schedule: Optional[ScheduleDuration] = None
    """
    The schedule that the customer will gain access to the credits purposed with
    this commit.
    """

    amount: Optional[float] = None
    """(DEPRECATED) Use access_schedule + invoice_schedule instead."""

    applicable_contract_ids: Optional[List[str]] = None

    applicable_product_ids: Optional[List[str]] = None

    applicable_product_tags: Optional[List[str]] = None

    archived_at: Optional[datetime] = None
    """RFC 3339 timestamp indicating when the commit was archived.

    If not provided, the commit is not archived.
    """

    balance: Optional[float] = None
    """The current balance of the credit or commit.

    This balance reflects the amount of credit or commit that the customer has
    access to use at this moment - thus, expired and upcoming credit or commit
    segments contribute 0 to the balance. The balance will match the sum of all
    ledger entries with the exception of the case where the sum of negative manual
    ledger entries exceeds the positive amount remaining on the credit or commit -
    in that case, the balance will be 0. All manual ledger entries associated with
    active credit or commit segments are included in the balance, including
    future-dated manual ledger entries.
    """

    contract: Optional[Contract] = None

    custom_fields: Optional[Dict[str, str]] = None

    description: Optional[str] = None

    hierarchy_configuration: Optional[HierarchyConfiguration] = None
    """Optional configuration for commit hierarchy access control"""

    invoice_contract: Optional[InvoiceContract] = None
    """The contract that this commit will be billed on."""

    invoice_schedule: Optional[SchedulePointInTime] = None
    """The schedule that the customer will be invoiced for this commit."""

    ledger: Optional[List[Ledger]] = None
    """A list of ordered events that impact the balance of a commit.

    For example, an invoice deduction or a rollover.
    """

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    priority: Optional[float] = None
    """
    If multiple credits or commits are applicable, the one with the lower priority
    will apply first.
    """

    rate_type: Optional[Literal["COMMIT_RATE", "LIST_RATE"]] = None

    rolled_over_from: Optional[RolledOverFrom] = None

    rollover_fraction: Optional[float] = None

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    specifiers: Optional[List[Specifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown.
    """

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a commit or credit is made with a uniqueness key that was
    previously used to create a commit or credit, a new record will not be created
    and the request will fail with a 409 error.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    ID as ID,
    Rate as Rate,
    Tier as Tier,
    Commit as Commit,
    Credit as Credit,
    Discount as Discount,
    Override as Override,
    ProService as ProService,
    CreditTypeData as CreditTypeData,
    PropertyFilter as PropertyFilter,
    BaseUsageFilter as BaseUsageFilter,
    EventTypeFilter as EventTypeFilter,
    ScheduledCharge as ScheduledCharge,
    ScheduleDuration as ScheduleDuration,
    SchedulePointInTime as SchedulePointInTime,
    ContractWithoutAmendments as ContractWithoutAmendments,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["InvoiceRetrieveParams"]


class InvoiceRetrieveParams(TypedDict, total=False):
    customer_id: Required[str]

    invoice_id: Required[str]

    skip_zero_qty_line_items: bool
    """If set, all zero quantity line items will be filtered out of the response"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from ..shared.id import ID

__all__ = ["BillableMetricCreateResponse"]


class BillableMetricCreateResponse(BaseModel):
    data: ID



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo
from .quantity_rounding_param import QuantityRoundingParam
from .quantity_conversion_param import QuantityConversionParam

__all__ = ["ProductUpdateParams"]


class ProductUpdateParams(TypedDict, total=False):
    product_id: Required[str]
    """ID of the product to update"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Timestamp representing when the update should go into effect.

    It must be on an hour boundary (e.g. 1:00, not 1:30).
    """

    billable_metric_id: str
    """Available for USAGE products only.

    If not provided, defaults to product's current billable metric.
    """

    composite_product_ids: List[str]
    """Available for COMPOSITE products only.

    If not provided, defaults to product's current composite_product_ids.
    """

    composite_tags: List[str]
    """Available for COMPOSITE products only.

    If not provided, defaults to product's current composite_tags.
    """

    exclude_free_usage: bool
    """Beta feature only available for composite products.

    If true, products with $0 will not be included when computing composite usage.
    Defaults to false
    """

    is_refundable: bool
    """Defaults to product's current refundability status.

    This field's availability is dependent on your client's configuration.
    """

    name: str
    """displayed on invoices. If not provided, defaults to product's current name."""

    netsuite_internal_item_id: str
    """If not provided, defaults to product's current netsuite_internal_item_id.

    This field's availability is dependent on your client's configuration.
    """

    netsuite_overage_item_id: str
    """Available for USAGE and COMPOSITE products only.

    If not provided, defaults to product's current netsuite_overage_item_id. This
    field's availability is dependent on your client's configuration.
    """

    presentation_group_key: List[str]
    """For USAGE products only.

    Groups usage line items on invoices. The superset of values in the pricing group
    key and presentation group key must be set as one compound group key on the
    billable metric.
    """

    pricing_group_key: List[str]
    """For USAGE products only.

    If set, pricing for this product will be determined for each pricing_group_key
    value, as opposed to the product as a whole. The superset of values in the
    pricing group key and presentation group key must be set as one compound group
    key on the billable metric.
    """

    quantity_conversion: Optional[QuantityConversionParam]
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be converted using
    the provided conversion factor and operation. For example, if the operation is
    "multiply" and the conversion factor is 100, then the quantity will be
    multiplied by 100. This can be used in cases where data is sent in one unit and
    priced in another. For example, data could be sent in MB and priced in GB. In
    this case, the conversion factor would be 1024 and the operation would be
    "divide".
    """

    quantity_rounding: Optional[QuantityRoundingParam]
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be rounded using
    the provided rounding method and decimal places. For example, if the method is
    "round up" and the decimal places is 0, then the quantity will be rounded up to
    the nearest integer.
    """

    tags: List[str]
    """If not provided, defaults to product's current tags"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .alerts import (
    AlertsResource,
    AsyncAlertsResource,
    AlertsResourceWithRawResponse,
    AsyncAlertsResourceWithRawResponse,
    AlertsResourceWithStreamingResponse,
    AsyncAlertsResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .services import (
    ServicesResource,
    AsyncServicesResource,
    ServicesResourceWithRawResponse,
    AsyncServicesResourceWithRawResponse,
    ServicesResourceWithStreamingResponse,
    AsyncServicesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .audit_logs import (
    AuditLogsResource,
    AsyncAuditLogsResource,
    AuditLogsResourceWithRawResponse,
    AsyncAuditLogsResourceWithRawResponse,
    AuditLogsResourceWithStreamingResponse,
    AsyncAuditLogsResourceWithStreamingResponse,
)
from .dashboards import (
    DashboardsResource,
    AsyncDashboardsResource,
    DashboardsResourceWithRawResponse,
    AsyncDashboardsResourceWithRawResponse,
    DashboardsResourceWithStreamingResponse,
    AsyncDashboardsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .credit_grants import (
    CreditGrantsResource,
    AsyncCreditGrantsResource,
    CreditGrantsResourceWithRawResponse,
    AsyncCreditGrantsResourceWithRawResponse,
    CreditGrantsResourceWithStreamingResponse,
    AsyncCreditGrantsResourceWithStreamingResponse,
)
from .custom_fields import (
    CustomFieldsResource,
    AsyncCustomFieldsResource,
    CustomFieldsResourceWithRawResponse,
    AsyncCustomFieldsResourceWithRawResponse,
    CustomFieldsResourceWithStreamingResponse,
    AsyncCustomFieldsResourceWithStreamingResponse,
)
from .pricing_units import (
    PricingUnitsResource,
    AsyncPricingUnitsResource,
    PricingUnitsResourceWithRawResponse,
    AsyncPricingUnitsResourceWithRawResponse,
    PricingUnitsResourceWithStreamingResponse,
    AsyncPricingUnitsResourceWithStreamingResponse,
)
from .billable_metrics import (
    BillableMetricsResource,
    AsyncBillableMetricsResource,
    BillableMetricsResourceWithRawResponse,
    AsyncBillableMetricsResourceWithRawResponse,
    BillableMetricsResourceWithStreamingResponse,
    AsyncBillableMetricsResourceWithStreamingResponse,
)
from .contracts.contracts import (
    ContractsResource,
    AsyncContractsResource,
    ContractsResourceWithRawResponse,
    AsyncContractsResourceWithRawResponse,
    ContractsResourceWithStreamingResponse,
    AsyncContractsResourceWithStreamingResponse,
)
from .customers.customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    @cached_property
    def alerts(self) -> AlertsResource:
        return AlertsResource(self._client)

    @cached_property
    def plans(self) -> PlansResource:
        return PlansResource(self._client)

    @cached_property
    def credit_grants(self) -> CreditGrantsResource:
        return CreditGrantsResource(self._client)

    @cached_property
    def pricing_units(self) -> PricingUnitsResource:
        return PricingUnitsResource(self._client)

    @cached_property
    def customers(self) -> CustomersResource:
        return CustomersResource(self._client)

    @cached_property
    def dashboards(self) -> DashboardsResource:
        return DashboardsResource(self._client)

    @cached_property
    def usage(self) -> UsageResource:
        return UsageResource(self._client)

    @cached_property
    def audit_logs(self) -> AuditLogsResource:
        return AuditLogsResource(self._client)

    @cached_property
    def custom_fields(self) -> CustomFieldsResource:
        return CustomFieldsResource(self._client)

    @cached_property
    def billable_metrics(self) -> BillableMetricsResource:
        return BillableMetricsResource(self._client)

    @cached_property
    def services(self) -> ServicesResource:
        return ServicesResource(self._client)

    @cached_property
    def invoices(self) -> InvoicesResource:
        return InvoicesResource(self._client)

    @cached_property
    def contracts(self) -> ContractsResource:
        return ContractsResource(self._client)

    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)


class AsyncV1Resource(AsyncAPIResource):
    @cached_property
    def alerts(self) -> AsyncAlertsResource:
        return AsyncAlertsResource(self._client)

    @cached_property
    def plans(self) -> AsyncPlansResource:
        return AsyncPlansResource(self._client)

    @cached_property
    def credit_grants(self) -> AsyncCreditGrantsResource:
        return AsyncCreditGrantsResource(self._client)

    @cached_property
    def pricing_units(self) -> AsyncPricingUnitsResource:
        return AsyncPricingUnitsResource(self._client)

    @cached_property
    def customers(self) -> AsyncCustomersResource:
        return AsyncCustomersResource(self._client)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResource:
        return AsyncDashboardsResource(self._client)

    @cached_property
    def usage(self) -> AsyncUsageResource:
        return AsyncUsageResource(self._client)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResource:
        return AsyncAuditLogsResource(self._client)

    @cached_property
    def custom_fields(self) -> AsyncCustomFieldsResource:
        return AsyncCustomFieldsResource(self._client)

    @cached_property
    def billable_metrics(self) -> AsyncBillableMetricsResource:
        return AsyncBillableMetricsResource(self._client)

    @cached_property
    def services(self) -> AsyncServicesResource:
        return AsyncServicesResource(self._client)

    @cached_property
    def invoices(self) -> AsyncInvoicesResource:
        return AsyncInvoicesResource(self._client)

    @cached_property
    def contracts(self) -> AsyncContractsResource:
        return AsyncContractsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def alerts(self) -> AlertsResourceWithRawResponse:
        return AlertsResourceWithRawResponse(self._v1.alerts)

    @cached_property
    def plans(self) -> PlansResourceWithRawResponse:
        return PlansResourceWithRawResponse(self._v1.plans)

    @cached_property
    def credit_grants(self) -> CreditGrantsResourceWithRawResponse:
        return CreditGrantsResourceWithRawResponse(self._v1.credit_grants)

    @cached_property
    def pricing_units(self) -> PricingUnitsResourceWithRawResponse:
        return PricingUnitsResourceWithRawResponse(self._v1.pricing_units)

    @cached_property
    def customers(self) -> CustomersResourceWithRawResponse:
        return CustomersResourceWithRawResponse(self._v1.customers)

    @cached_property
    def dashboards(self) -> DashboardsResourceWithRawResponse:
        return DashboardsResourceWithRawResponse(self._v1.dashboards)

    @cached_property
    def usage(self) -> UsageResourceWithRawResponse:
        return UsageResourceWithRawResponse(self._v1.usage)

    @cached_property
    def audit_logs(self) -> AuditLogsResourceWithRawResponse:
        return AuditLogsResourceWithRawResponse(self._v1.audit_logs)

    @cached_property
    def custom_fields(self) -> CustomFieldsResourceWithRawResponse:
        return CustomFieldsResourceWithRawResponse(self._v1.custom_fields)

    @cached_property
    def billable_metrics(self) -> BillableMetricsResourceWithRawResponse:
        return BillableMetricsResourceWithRawResponse(self._v1.billable_metrics)

    @cached_property
    def services(self) -> ServicesResourceWithRawResponse:
        return ServicesResourceWithRawResponse(self._v1.services)

    @cached_property
    def invoices(self) -> InvoicesResourceWithRawResponse:
        return InvoicesResourceWithRawResponse(self._v1.invoices)

    @cached_property
    def contracts(self) -> ContractsResourceWithRawResponse:
        return ContractsResourceWithRawResponse(self._v1.contracts)


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def alerts(self) -> AsyncAlertsResourceWithRawResponse:
        return AsyncAlertsResourceWithRawResponse(self._v1.alerts)

    @cached_property
    def plans(self) -> AsyncPlansResourceWithRawResponse:
        return AsyncPlansResourceWithRawResponse(self._v1.plans)

    @cached_property
    def credit_grants(self) -> AsyncCreditGrantsResourceWithRawResponse:
        return AsyncCreditGrantsResourceWithRawResponse(self._v1.credit_grants)

    @cached_property
    def pricing_units(self) -> AsyncPricingUnitsResourceWithRawResponse:
        return AsyncPricingUnitsResourceWithRawResponse(self._v1.pricing_units)

    @cached_property
    def customers(self) -> AsyncCustomersResourceWithRawResponse:
        return AsyncCustomersResourceWithRawResponse(self._v1.customers)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResourceWithRawResponse:
        return AsyncDashboardsResourceWithRawResponse(self._v1.dashboards)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithRawResponse:
        return AsyncUsageResourceWithRawResponse(self._v1.usage)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResourceWithRawResponse:
        return AsyncAuditLogsResourceWithRawResponse(self._v1.audit_logs)

    @cached_property
    def custom_fields(self) -> AsyncCustomFieldsResourceWithRawResponse:
        return AsyncCustomFieldsResourceWithRawResponse(self._v1.custom_fields)

    @cached_property
    def billable_metrics(self) -> AsyncBillableMetricsResourceWithRawResponse:
        return AsyncBillableMetricsResourceWithRawResponse(self._v1.billable_metrics)

    @cached_property
    def services(self) -> AsyncServicesResourceWithRawResponse:
        return AsyncServicesResourceWithRawResponse(self._v1.services)

    @cached_property
    def invoices(self) -> AsyncInvoicesResourceWithRawResponse:
        return AsyncInvoicesResourceWithRawResponse(self._v1.invoices)

    @cached_property
    def contracts(self) -> AsyncContractsResourceWithRawResponse:
        return AsyncContractsResourceWithRawResponse(self._v1.contracts)


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def alerts(self) -> AlertsResourceWithStreamingResponse:
        return AlertsResourceWithStreamingResponse(self._v1.alerts)

    @cached_property
    def plans(self) -> PlansResourceWithStreamingResponse:
        return PlansResourceWithStreamingResponse(self._v1.plans)

    @cached_property
    def credit_grants(self) -> CreditGrantsResourceWithStreamingResponse:
        return CreditGrantsResourceWithStreamingResponse(self._v1.credit_grants)

    @cached_property
    def pricing_units(self) -> PricingUnitsResourceWithStreamingResponse:
        return PricingUnitsResourceWithStreamingResponse(self._v1.pricing_units)

    @cached_property
    def customers(self) -> CustomersResourceWithStreamingResponse:
        return CustomersResourceWithStreamingResponse(self._v1.customers)

    @cached_property
    def dashboards(self) -> DashboardsResourceWithStreamingResponse:
        return DashboardsResourceWithStreamingResponse(self._v1.dashboards)

    @cached_property
    def usage(self) -> UsageResourceWithStreamingResponse:
        return UsageResourceWithStreamingResponse(self._v1.usage)

    @cached_property
    def audit_logs(self) -> AuditLogsResourceWithStreamingResponse:
        return AuditLogsResourceWithStreamingResponse(self._v1.audit_logs)

    @cached_property
    def custom_fields(self) -> CustomFieldsResourceWithStreamingResponse:
        return CustomFieldsResourceWithStreamingResponse(self._v1.custom_fields)

    @cached_property
    def billable_metrics(self) -> BillableMetricsResourceWithStreamingResponse:
        return BillableMetricsResourceWithStreamingResponse(self._v1.billable_metrics)

    @cached_property
    def services(self) -> ServicesResourceWithStreamingResponse:
        return ServicesResourceWithStreamingResponse(self._v1.services)

    @cached_property
    def invoices(self) -> InvoicesResourceWithStreamingResponse:
        return InvoicesResourceWithStreamingResponse(self._v1.invoices)

    @cached_property
    def contracts(self) -> ContractsResourceWithStreamingResponse:
        return ContractsResourceWithStreamingResponse(self._v1.contracts)


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def alerts(self) -> AsyncAlertsResourceWithStreamingResponse:
        return AsyncAlertsResourceWithStreamingResponse(self._v1.alerts)

    @cached_property
    def plans(self) -> AsyncPlansResourceWithStreamingResponse:
        return AsyncPlansResourceWithStreamingResponse(self._v1.plans)

    @cached_property
    def credit_grants(self) -> AsyncCreditGrantsResourceWithStreamingResponse:
        return AsyncCreditGrantsResourceWithStreamingResponse(self._v1.credit_grants)

    @cached_property
    def pricing_units(self) -> AsyncPricingUnitsResourceWithStreamingResponse:
        return AsyncPricingUnitsResourceWithStreamingResponse(self._v1.pricing_units)

    @cached_property
    def customers(self) -> AsyncCustomersResourceWithStreamingResponse:
        return AsyncCustomersResourceWithStreamingResponse(self._v1.customers)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResourceWithStreamingResponse:
        return AsyncDashboardsResourceWithStreamingResponse(self._v1.dashboards)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithStreamingResponse:
        return AsyncUsageResourceWithStreamingResponse(self._v1.usage)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        return AsyncAuditLogsResourceWithStreamingResponse(self._v1.audit_logs)

    @cached_property
    def custom_fields(self) -> AsyncCustomFieldsResourceWithStreamingResponse:
        return AsyncCustomFieldsResourceWithStreamingResponse(self._v1.custom_fields)

    @cached_property
    def billable_metrics(self) -> AsyncBillableMetricsResourceWithStreamingResponse:
        return AsyncBillableMetricsResourceWithStreamingResponse(self._v1.billable_metrics)

    @cached_property
    def services(self) -> AsyncServicesResourceWithStreamingResponse:
        return AsyncServicesResourceWithStreamingResponse(self._v1.services)

    @cached_property
    def invoices(self) -> AsyncInvoicesResourceWithStreamingResponse:
        return AsyncInvoicesResourceWithStreamingResponse(self._v1.invoices)

    @cached_property
    def contracts(self) -> AsyncContractsResourceWithStreamingResponse:
        return AsyncContractsResourceWithStreamingResponse(self._v1.contracts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel
from .schedule_point_in_time import SchedulePointInTime

__all__ = ["Discount", "Product"]


class Product(BaseModel):
    id: str

    name: str


class Discount(BaseModel):
    id: str

    product: Product

    schedule: SchedulePointInTime

    custom_fields: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import (
    credit_grant_edit_params,
    credit_grant_list_params,
    credit_grant_void_params,
    credit_grant_create_params,
    credit_grant_list_entries_params,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.credit_grant_edit_response import CreditGrantEditResponse
from ...types.v1.credit_grant_list_response import CreditGrantListResponse
from ...types.v1.credit_grant_void_response import CreditGrantVoidResponse
from ...types.v1.credit_grant_create_response import CreditGrantCreateResponse
from ...types.v1.credit_grant_list_entries_response import CreditGrantListEntriesResponse

__all__ = ["CreditGrantsResource", "AsyncCreditGrantsResource"]


class CreditGrantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CreditGrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return CreditGrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditGrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return CreditGrantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        customer_id: str,
        expires_at: Union[str, datetime],
        grant_amount: credit_grant_create_params.GrantAmount,
        name: str,
        paid_amount: credit_grant_create_params.PaidAmount,
        priority: float,
        credit_grant_type: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        invoice_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        reason: str | NotGiven = NOT_GIVEN,
        rollover_settings: credit_grant_create_params.RolloverSettings | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantCreateResponse:
        """
        Create a new credit grant

        Args:
          customer_id: the Metronome ID of the customer

          expires_at: The credit grant will only apply to usage or charges dated before this timestamp

          grant_amount: the amount of credits granted

          name: the name of the credit grant as it will appear on invoices

          paid_amount: the amount paid for this credit grant

          custom_fields: Custom fields to attach to the credit grant.

          effective_at: The credit grant will only apply to usage or charges dated on or after this
              timestamp

          invoice_date: The date to issue an invoice for the paid_amount.

          product_ids: The product(s) which these credits will be applied to. (If unspecified, the
              credits will be applied to charges for all products.). The array ordering
              specified here will be used to determine the order in which credits will be
              applied to invoice line items

          rollover_settings: Configure a rollover for this credit grant so if it expires it rolls over a
              configured amount to a new credit grant. This feature is currently opt-in only.
              Contact Metronome to be added to the beta.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/credits/createGrant",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "expires_at": expires_at,
                    "grant_amount": grant_amount,
                    "name": name,
                    "paid_amount": paid_amount,
                    "priority": priority,
                    "credit_grant_type": credit_grant_type,
                    "custom_fields": custom_fields,
                    "effective_at": effective_at,
                    "invoice_date": invoice_date,
                    "product_ids": product_ids,
                    "reason": reason,
                    "rollover_settings": rollover_settings,
                    "uniqueness_key": uniqueness_key,
                },
                credit_grant_create_params.CreditGrantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantCreateResponse,
        )

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        credit_grant_ids: List[str] | NotGiven = NOT_GIVEN,
        credit_type_ids: List[str] | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        effective_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        not_expiring_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[CreditGrantListResponse]:
        """List credit grants.

        This list does not included voided grants.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          credit_grant_ids: An array of credit grant IDs. If this is specified, neither credit_type_ids nor
              customer_ids may be specified.

          credit_type_ids: An array of credit type IDs. This must not be specified if credit_grant_ids is
              specified.

          customer_ids: An array of Metronome customer IDs. This must not be specified if
              credit_grant_ids is specified.

          effective_before: Only return credit grants that are effective before this timestamp (exclusive).

          not_expiring_before: Only return credit grants that expire at or after this timestamp.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/credits/listGrants",
            page=SyncCursorPage[CreditGrantListResponse],
            body=maybe_transform(
                {
                    "credit_grant_ids": credit_grant_ids,
                    "credit_type_ids": credit_type_ids,
                    "customer_ids": customer_ids,
                    "effective_before": effective_before,
                    "not_expiring_before": not_expiring_before,
                },
                credit_grant_list_params.CreditGrantListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    credit_grant_list_params.CreditGrantListParams,
                ),
            ),
            model=CreditGrantListResponse,
            method="post",
        )

    def edit(
        self,
        *,
        id: str,
        credit_grant_type: str | NotGiven = NOT_GIVEN,
        expires_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantEditResponse:
        """
        Edit an existing credit grant

        Args:
          id: the ID of the credit grant

          credit_grant_type: the updated credit grant type

          expires_at: the updated expiration date for the credit grant

          name: the updated name for the credit grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/credits/editGrant",
            body=maybe_transform(
                {
                    "id": id,
                    "credit_grant_type": credit_grant_type,
                    "expires_at": expires_at,
                    "name": name,
                },
                credit_grant_edit_params.CreditGrantEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantEditResponse,
        )

    def list_entries(
        self,
        *,
        next_page: str | NotGiven = NOT_GIVEN,
        sort: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        credit_type_ids: List[str] | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantListEntriesResponse:
        """Fetches a list of credit ledger entries.

        Returns lists of ledgers per customer.
        Ledger entries are returned in chronological order. Ledger entries associated
        with voided credit grants are not included.

        Args:
          next_page: Cursor that indicates where the next page of results should start.

          sort: Ledgers sort order by date, asc or desc. Defaults to asc.

          credit_type_ids: A list of Metronome credit type IDs to fetch ledger entries for. If absent,
              ledger entries for all credit types will be returned.

          customer_ids: A list of Metronome customer IDs to fetch ledger entries for. If absent, ledger
              entries for all customers will be returned.

          ending_before: If supplied, ledger entries will only be returned with an effective_at before
              this time. This timestamp must not be in the future. If no timestamp is
              supplied, all entries up to the start of the customer's next billing period will
              be returned.

          starting_on: If supplied, only ledger entries effective at or after this time will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/credits/listEntries",
            body=maybe_transform(
                {
                    "credit_type_ids": credit_type_ids,
                    "customer_ids": customer_ids,
                    "ending_before": ending_before,
                    "starting_on": starting_on,
                },
                credit_grant_list_entries_params.CreditGrantListEntriesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_page": next_page,
                        "sort": sort,
                    },
                    credit_grant_list_entries_params.CreditGrantListEntriesParams,
                ),
            ),
            cast_to=CreditGrantListEntriesResponse,
        )

    def void(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | NotGiven = NOT_GIVEN,
        void_credit_purchase_invoice: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantVoidResponse:
        """
        Void a credit grant

        Args:
          release_uniqueness_key: If true, resets the uniqueness key on this grant so it can be re-used

          void_credit_purchase_invoice: If true, void the purchase invoice associated with the grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/credits/voidGrant",
            body=maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                    "void_credit_purchase_invoice": void_credit_purchase_invoice,
                },
                credit_grant_void_params.CreditGrantVoidParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantVoidResponse,
        )


class AsyncCreditGrantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCreditGrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditGrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditGrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncCreditGrantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        customer_id: str,
        expires_at: Union[str, datetime],
        grant_amount: credit_grant_create_params.GrantAmount,
        name: str,
        paid_amount: credit_grant_create_params.PaidAmount,
        priority: float,
        credit_grant_type: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        invoice_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        reason: str | NotGiven = NOT_GIVEN,
        rollover_settings: credit_grant_create_params.RolloverSettings | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantCreateResponse:
        """
        Create a new credit grant

        Args:
          customer_id: the Metronome ID of the customer

          expires_at: The credit grant will only apply to usage or charges dated before this timestamp

          grant_amount: the amount of credits granted

          name: the name of the credit grant as it will appear on invoices

          paid_amount: the amount paid for this credit grant

          custom_fields: Custom fields to attach to the credit grant.

          effective_at: The credit grant will only apply to usage or charges dated on or after this
              timestamp

          invoice_date: The date to issue an invoice for the paid_amount.

          product_ids: The product(s) which these credits will be applied to. (If unspecified, the
              credits will be applied to charges for all products.). The array ordering
              specified here will be used to determine the order in which credits will be
              applied to invoice line items

          rollover_settings: Configure a rollover for this credit grant so if it expires it rolls over a
              configured amount to a new credit grant. This feature is currently opt-in only.
              Contact Metronome to be added to the beta.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/credits/createGrant",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "expires_at": expires_at,
                    "grant_amount": grant_amount,
                    "name": name,
                    "paid_amount": paid_amount,
                    "priority": priority,
                    "credit_grant_type": credit_grant_type,
                    "custom_fields": custom_fields,
                    "effective_at": effective_at,
                    "invoice_date": invoice_date,
                    "product_ids": product_ids,
                    "reason": reason,
                    "rollover_settings": rollover_settings,
                    "uniqueness_key": uniqueness_key,
                },
                credit_grant_create_params.CreditGrantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantCreateResponse,
        )

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        credit_grant_ids: List[str] | NotGiven = NOT_GIVEN,
        credit_type_ids: List[str] | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        effective_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        not_expiring_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[CreditGrantListResponse, AsyncCursorPage[CreditGrantListResponse]]:
        """List credit grants.

        This list does not included voided grants.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          credit_grant_ids: An array of credit grant IDs. If this is specified, neither credit_type_ids nor
              customer_ids may be specified.

          credit_type_ids: An array of credit type IDs. This must not be specified if credit_grant_ids is
              specified.

          customer_ids: An array of Metronome customer IDs. This must not be specified if
              credit_grant_ids is specified.

          effective_before: Only return credit grants that are effective before this timestamp (exclusive).

          not_expiring_before: Only return credit grants that expire at or after this timestamp.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/credits/listGrants",
            page=AsyncCursorPage[CreditGrantListResponse],
            body=maybe_transform(
                {
                    "credit_grant_ids": credit_grant_ids,
                    "credit_type_ids": credit_type_ids,
                    "customer_ids": customer_ids,
                    "effective_before": effective_before,
                    "not_expiring_before": not_expiring_before,
                },
                credit_grant_list_params.CreditGrantListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    credit_grant_list_params.CreditGrantListParams,
                ),
            ),
            model=CreditGrantListResponse,
            method="post",
        )

    async def edit(
        self,
        *,
        id: str,
        credit_grant_type: str | NotGiven = NOT_GIVEN,
        expires_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantEditResponse:
        """
        Edit an existing credit grant

        Args:
          id: the ID of the credit grant

          credit_grant_type: the updated credit grant type

          expires_at: the updated expiration date for the credit grant

          name: the updated name for the credit grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/credits/editGrant",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "credit_grant_type": credit_grant_type,
                    "expires_at": expires_at,
                    "name": name,
                },
                credit_grant_edit_params.CreditGrantEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantEditResponse,
        )

    async def list_entries(
        self,
        *,
        next_page: str | NotGiven = NOT_GIVEN,
        sort: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        credit_type_ids: List[str] | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantListEntriesResponse:
        """Fetches a list of credit ledger entries.

        Returns lists of ledgers per customer.
        Ledger entries are returned in chronological order. Ledger entries associated
        with voided credit grants are not included.

        Args:
          next_page: Cursor that indicates where the next page of results should start.

          sort: Ledgers sort order by date, asc or desc. Defaults to asc.

          credit_type_ids: A list of Metronome credit type IDs to fetch ledger entries for. If absent,
              ledger entries for all credit types will be returned.

          customer_ids: A list of Metronome customer IDs to fetch ledger entries for. If absent, ledger
              entries for all customers will be returned.

          ending_before: If supplied, ledger entries will only be returned with an effective_at before
              this time. This timestamp must not be in the future. If no timestamp is
              supplied, all entries up to the start of the customer's next billing period will
              be returned.

          starting_on: If supplied, only ledger entries effective at or after this time will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/credits/listEntries",
            body=await async_maybe_transform(
                {
                    "credit_type_ids": credit_type_ids,
                    "customer_ids": customer_ids,
                    "ending_before": ending_before,
                    "starting_on": starting_on,
                },
                credit_grant_list_entries_params.CreditGrantListEntriesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "next_page": next_page,
                        "sort": sort,
                    },
                    credit_grant_list_entries_params.CreditGrantListEntriesParams,
                ),
            ),
            cast_to=CreditGrantListEntriesResponse,
        )

    async def void(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | NotGiven = NOT_GIVEN,
        void_credit_purchase_invoice: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditGrantVoidResponse:
        """
        Void a credit grant

        Args:
          release_uniqueness_key: If true, resets the uniqueness key on this grant so it can be re-used

          void_credit_purchase_invoice: If true, void the purchase invoice associated with the grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/credits/voidGrant",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                    "void_credit_purchase_invoice": void_credit_purchase_invoice,
                },
                credit_grant_void_params.CreditGrantVoidParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantVoidResponse,
        )


class CreditGrantsResourceWithRawResponse:
    def __init__(self, credit_grants: CreditGrantsResource) -> None:
        self._credit_grants = credit_grants

        self.create = to_raw_response_wrapper(
            credit_grants.create,
        )
        self.list = to_raw_response_wrapper(
            credit_grants.list,
        )
        self.edit = to_raw_response_wrapper(
            credit_grants.edit,
        )
        self.list_entries = to_raw_response_wrapper(
            credit_grants.list_entries,
        )
        self.void = to_raw_response_wrapper(
            credit_grants.void,
        )


class AsyncCreditGrantsResourceWithRawResponse:
    def __init__(self, credit_grants: AsyncCreditGrantsResource) -> None:
        self._credit_grants = credit_grants

        self.create = async_to_raw_response_wrapper(
            credit_grants.create,
        )
        self.list = async_to_raw_response_wrapper(
            credit_grants.list,
        )
        self.edit = async_to_raw_response_wrapper(
            credit_grants.edit,
        )
        self.list_entries = async_to_raw_response_wrapper(
            credit_grants.list_entries,
        )
        self.void = async_to_raw_response_wrapper(
            credit_grants.void,
        )


class CreditGrantsResourceWithStreamingResponse:
    def __init__(self, credit_grants: CreditGrantsResource) -> None:
        self._credit_grants = credit_grants

        self.create = to_streamed_response_wrapper(
            credit_grants.create,
        )
        self.list = to_streamed_response_wrapper(
            credit_grants.list,
        )
        self.edit = to_streamed_response_wrapper(
            credit_grants.edit,
        )
        self.list_entries = to_streamed_response_wrapper(
            credit_grants.list_entries,
        )
        self.void = to_streamed_response_wrapper(
            credit_grants.void,
        )


class AsyncCreditGrantsResourceWithStreamingResponse:
    def __init__(self, credit_grants: AsyncCreditGrantsResource) -> None:
        self._credit_grants = credit_grants

        self.create = async_to_streamed_response_wrapper(
            credit_grants.create,
        )
        self.list = async_to_streamed_response_wrapper(
            credit_grants.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            credit_grants.edit,
        )
        self.list_entries = async_to_streamed_response_wrapper(
            credit_grants.list_entries,
        )
        self.void = async_to_streamed_response_wrapper(
            credit_grants.void,
        )
